package com.jiayue.ssi.controller;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jiayue.ssi.annotation.InterfaceLimit;
import com.jiayue.ssi.annotation.OperateLog;
import com.jiayue.ssi.backenum.AuditType;
import com.jiayue.ssi.backenum.BusinessType;
import com.jiayue.ssi.entity.SysLogininfor;
import com.jiayue.ssi.service.SysLogininforService;
import com.jiayue.ssi.util.DateUtils;
import com.jiayue.ssi.util.ResponseVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

/**
 * 系统访问记录
 *
 * @author ruoyi
 */
@RestController
@RequestMapping("/sysLogininforController")
@Slf4j
public class SysLogininforController {
    @Autowired
    SysLogininforService sysLogininforService;

    /**
     * 获取列表分页信息
     *
     * @return 列表信息
     */
    @GetMapping(value = "/getAll")
    @PreAuthorize("@ss.hasPermi('auditManager:logininfor:list')")
    public ResponseVO getAll(Integer currentPage, Integer pageSize, String ipaddr, String userName,
                             String status,String startLoginTime,String endLoginTime) {
        try {
            if (StringUtils.isNotEmpty(ipaddr)) {
                if (ipaddr.length() > 128) {
                    return ResponseVO.fail("登录地址长度不能超过128个字符！");
                }
            }
            if (StringUtils.isNotEmpty(userName)) {
                if (userName.length() > 50) {
                    return ResponseVO.fail("用户名称长度不能超过50个字符！");
                }
            }
            QueryWrapper<SysLogininfor> wrapper = new QueryWrapper<>();
            if (StringUtils.isNotEmpty(ipaddr)) {
                wrapper.eq("ipaddr", ipaddr);
            }
            if (StringUtils.isNotEmpty(userName)) {
                wrapper.eq("user_name", userName);
            }
            if (StringUtils.isNotEmpty(status)) {
                wrapper.eq("status", status);
            }
            if (StringUtils.isNotEmpty(startLoginTime)) {
                wrapper.ge("login_time", DateUtils.getDayStartTime(DateUtil.parseDate(startLoginTime)));
            }
            if (StringUtils.isNotEmpty(endLoginTime)) {
                wrapper.le("login_time", DateUtils.getDayLastTime(DateUtil.parseDate(endLoginTime)));
            }
            Page<SysLogininfor> result = sysLogininforService.page(new Page<>(currentPage, pageSize), wrapper);
            return ResponseVO.success(result);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("获取登录日志列表异常");
            return ResponseVO.error(null);
        }
    }

    /**
     * 删除登录信息
     */
    @PostMapping(value = "/delLoginInfo")
    @OperateLog(title = "登录日志", businessType = BusinessType.DELETE,auditType = AuditType.SYS)
    @PreAuthorize("@ss.hasPermi('auditManager:logininfor:remove')")
    public ResponseVO delLoginInfo(String infoId) {
        if (StringUtils.isEmpty(infoId)) {
            return ResponseVO.fail("id不能为空!");
        }
        try {
            boolean bo = sysLogininforService.removeLoginInfoById(Long.parseLong(infoId));
            if (bo) {
                return ResponseVO.success("删除登录信息成功");
            } else {
                log.error("删除用户信息失败");
                return ResponseVO.fail("删除登录信息失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("删除登录信息异常");
            return ResponseVO.error(e);
        }
    }
    /**
     * 清空登录信息
     */
    @PostMapping("/cleanLogininfor")
    @OperateLog(title = "登录日志", businessType = BusinessType.CLEAN,auditType = AuditType.SYS)
    @PreAuthorize("@ss.hasPermi('auditManager:logininfor:remove')")
    public ResponseVO cleanLogininfor() {
        try {
            boolean bo = sysLogininforService.cleanLogininfor();
            if (bo) {
                return ResponseVO.success("清空成功");
            } else {
                log.error("清空失败");
                return ResponseVO.fail("清空失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("清空异常");
            return ResponseVO.error(e);
        }
    }
}
