package com.jiayue.ssi.controller;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jiayue.ssi.annotation.InterfaceLimit;
import com.jiayue.ssi.annotation.OperateLog;
import com.jiayue.ssi.backenum.AuditType;
import com.jiayue.ssi.backenum.BusinessType;
import com.jiayue.ssi.entity.SysOperLog;
import com.jiayue.ssi.service.SysOperLogService;
import com.jiayue.ssi.util.DateUtils;
import com.jiayue.ssi.util.ResponseVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

/**
 * 操作日志记录
 *
 * @author ruoyi
 */
@RestController
@RequestMapping("/sysOperlogController")
@Slf4j
public class SysOperlogController {
    @Autowired
    SysOperLogService sysOperLogService;
    /**
     * 获取列表分页信息
     *
     * @return 列表信息
     */
    @GetMapping(value = "/getAll")
    @PreAuthorize("@ss.hasPermi('auditManager:operlog:list')")
    public ResponseVO getAll(Integer currentPage, Integer pageSize, String title, String operName,String auditType,
                             String businessType, String status,String startOperTime, String endOperTime) {
        try {
            if (StringUtils.isNotEmpty(title)) {
                if (title.length() > 50) {
                    return ResponseVO.fail("系统模块长度不能超过50个字符！");
                }
            }
            if (StringUtils.isNotEmpty(operName)) {
                if (operName.length() > 50) {
                    return ResponseVO.fail("操作人员长度不能超过50个字符！");
                }
            }
            QueryWrapper<SysOperLog> wrapper = new QueryWrapper<>();
            if (StringUtils.isNotEmpty(title)) {
                wrapper.like("title", title);
            }
            if (StringUtils.isNotEmpty(operName)) {
                wrapper.like("oper_name", operName);
            }
            if (StringUtils.isNotEmpty(auditType)) {
                wrapper.eq("audit_type", auditType);
            }
            if (StringUtils.isNotEmpty(businessType)) {
                wrapper.eq("business_type", businessType);
            }
            if (StringUtils.isNotEmpty(status)) {
                wrapper.eq("status", status);
            }
            if (StringUtils.isNotEmpty(startOperTime)) {
                wrapper.ge("oper_time", DateUtils.getDayStartTime(DateUtil.parseDate(startOperTime)));
            }
            if (StringUtils.isNotEmpty(endOperTime)) {
                wrapper.le("oper_time", DateUtils.getDayLastTime(DateUtil.parseDate(endOperTime)));
            }
            Page<SysOperLog> result = sysOperLogService.page(new Page<>(currentPage, pageSize), wrapper);
            return ResponseVO.success(result);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("获取操作日志列表异常");
            return ResponseVO.error(null);
        }
    }
    /**
     * 删除登录信息
     */
    @PostMapping(value = "/delOperlog")
    @OperateLog(title = "操作日志", businessType = BusinessType.DELETE,auditType = AuditType.SYS)
    @PreAuthorize("@ss.hasPermi('auditManager:operlog:remove')")
    public ResponseVO delOperlog(String operId) {
        if (StringUtils.isEmpty(operId)) {
            return ResponseVO.fail("id不能为空!");
        }
        try {
            boolean bo = sysOperLogService.removeOperlogById(Long.parseLong(operId));
            if (bo) {
                return ResponseVO.success("删除操作信息成功");
            } else {
                log.error("删除操作信息失败");
                return ResponseVO.fail("删除操作信息失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("删除操作信息异常");
            return ResponseVO.error(e);
        }
    }
    /**
     * 清空登录信息
     */
    @PostMapping("/cleanOperLog")
    @OperateLog(title = "操作日志", businessType = BusinessType.CLEAN,auditType = AuditType.SYS)
    @PreAuthorize("@ss.hasPermi('auditManager:operlog:remove')")
    public ResponseVO cleanOperLog() {
        try {
            boolean bo = sysOperLogService.cleanOperLog();
            if (bo) {
                return ResponseVO.success("清空成功");
            } else {
                log.error("清空失败");
                return ResponseVO.fail("清空失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("清空异常");
            return ResponseVO.error(e);
        }
    }
}
