package com.jiayue.ssi.handler;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.jiayue.ssi.constant.CacheConstants;
import com.jiayue.ssi.constant.Constants;
import com.jiayue.ssi.constant.LoginConstants;
import com.jiayue.ssi.entity.SysUser;
import com.jiayue.ssi.factory.LoginFactory;
import com.jiayue.ssi.service.SysParameterService;
import com.jiayue.ssi.service.SysUserService;
import com.jiayue.ssi.util.LocalCache;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.authentication.SimpleUrlAuthenticationFailureHandler;
import org.springframework.stereotype.Component;

/**
* 失败认证处理
* @author xsl
* @date 2023/2/16
*/
@Component("customAuthenticationFailureHandler")
public class CustomAuthenticationFailureHandler extends SimpleUrlAuthenticationFailureHandler {
    @Autowired
    SysUserService sysUserService;
    @Autowired
    SysParameterService sysParameterService;

    @Override
    public void onAuthenticationFailure(HttpServletRequest request, HttpServletResponse response,
        AuthenticationException e) throws IOException, ServletException {
        String errorTips = "";
        // 验证码验证
        String username = request.getParameter("username");
        SysUser user = sysUserService.queryUserName(username);
        if (user == null) {
            errorTips = "用户名或密码不正确";
        }
        else {
            // 失败总次数
            int errCount = 5;
            try {
                errCount = Integer.parseInt(sysParameterService.queryByKey("errCount", "5"));
            } catch (Exception errCountErr) {
                // 日志输出转换失败的提示
            }
            int lockedTime = 1200000;
            try {
                lockedTime = Integer.parseInt(sysParameterService.queryByKey("lockedTime", "1200000"));
            } catch (Exception lockedTimeErr) {
                // 日志输出转换失败的提示
            }
            if (user.getStatus().equals("1")) {
                Long xz = (user.getLockTime() + lockedTime - System.currentTimeMillis()) / 1000;
                if (xz < 0) {
                    if (errCount-1==0) {
                        errorTips = "用户锁定，稍后再试";
                        user.setErrNum(0);
                        user.setStatus("1");
                        user.setLockTime(System.currentTimeMillis());
                    }
                    else{
                        user.setStatus("0");
                        user.setErrNum(1);
                        errorTips = "用户名或密码不正确，还有"+(errCount-1)+"次机会";
                    }
                    sysUserService.updateUser(user);
                } else {
                    //锁定状态
                    errorTips = "用户已锁定,请隔" + xz + "秒再登录";
                }
            } else {
                int errNum = user.getErrNum();
                if (errNum < errCount-1) {
                    errorTips = "用户名或密码不正确，还有" + (errCount-1 - errNum) + "次机会";
                    errNum++;
                    user.setErrNum(errNum);
                    sysUserService.updateUser(user);
                } else {
                    errorTips = "用户锁定，稍后再试";
                    user.setErrNum(0);
                    user.setStatus("1");
                    user.setLockTime(System.currentTimeMillis());
                    sysUserService.updateUser(user);
                }
            }
        }
        // 记录用户退出日志
        LoginFactory.recordLogininfor(username, Constants.LOGIN_FAIL, "登录失败");

        // 删除缓存邮箱口令
        LocalCache.remove(CacheConstants.MAIL_CODE_KEY + username);
        // 清除
        response.addHeader("Access-Control-Allow-Origin", "*");
        response.setContentType("text/html;charset=UTF-8");
        response.setStatus(401);
        response.getWriter().write(errorTips);
    }

}
