package com.jiayue.ssi.handler;

import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.jiayue.ssi.constant.CacheConstants;
import com.jiayue.ssi.constant.Constants;
import com.jiayue.ssi.constant.SecretKeyConstants;
import com.jiayue.ssi.factory.LoginFactory;
import com.jiayue.ssi.service.SysUserService;
import com.jiayue.ssi.util.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.authentication.SavedRequestAwareAuthenticationSuccessHandler;
import org.springframework.stereotype.Component;

import com.jiayue.ssi.entity.SysUser;

import cn.hutool.json.JSONUtil;

/**
* 成功认证处理
* @author xsl
* @date 2023/2/16
*/
@Component("customAuthenticationSuccessHandler")
public class CustomAuthenticationSuccessHandler extends SavedRequestAwareAuthenticationSuccessHandler {
    @Autowired
    JwtTokenUtil jwtTokenUtil;
    @Autowired
    SysUserService sysUserService;

    @Override
    public void onAuthenticationSuccess(HttpServletRequest request, HttpServletResponse response, Authentication authentication) throws IOException, ServletException {
        // 验证码验证
        String username = request.getParameter("username");
        // 删除缓存邮箱口令
        LocalCache.remove(CacheConstants.MAIL_CODE_KEY + username);
        SysUser sysUser = (SysUser) authentication.getPrincipal();
        if (sysUser.getErrNum()>0){
            // 清除之前失败次数
            sysUser.setErrNum(0);
            sysUser.setLockTime(0L);
            sysUser.setStatus("0");
        }
        // 加入登录IP和时间
        sysUser.setLoginIp(IPUtils.getIpAddr(request));
        sysUser.setLoginDate(new Date());
        sysUserService.updateUser(sysUser);

        SecurityContextHolder.getContext().setAuthentication(authentication);

        // 记录用户退出日志
        LoginFactory.recordLogininfor(username, Constants.LOGIN_SUCCESS, "登录成功");

        String token = jwtTokenUtil.generateToken(sysUser);
        // 将token存储内存中，便于重复登录比对
        CacheConstants.LOGIN_TOKEN_MAP.put(username,token);

        String obj = JSONUtil.toJsonStr(ResponseVO.success(token));
        // token加密处理
        String encrypt = SM2CryptUtils.encrypt(obj, SecretKeyConstants.CLIENT_PUBLIC_KEY);
        // token签名
        String signStr = SM2CryptUtils.sign(SecretKeyConstants.SERVER_PRIVATE_KEY,obj);
        String secretResult = "token="+encrypt+"&tokenSign="+signStr;
        response.addHeader("Access-Control-Allow-Origin", "*");
        response.setStatus(200);
        response.setContentType("text/html;charset=UTF-8");
        response.getWriter().write(secretResult);
    }


}
