package com.jiayue.ssi.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jiayue.ssi.entity.SysUser;
import com.jiayue.ssi.mapper.SysUserMapper;
import com.jiayue.ssi.service.SysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;


import java.util.List;

/**
* 用户管理服务类
* @author xsl
* @date 2023/2/16
*/
@Service
public class SysUserServiceImpl extends ServiceImpl<SysUserMapper, SysUser> implements SysUserService {
    @Autowired
    SysUserMapper sysUserMapper;
    /**
     * 获取所有用户
     * @return List<SysUser>
     */
    @Override
    public List<SysUser> queryAllUsers(){
        return this.list();
    }
    /**
     * 保存用户
     * @param sysUser
     * @return boolean
     */
    @Override
    public boolean saveUser(SysUser sysUser){
       return this.save(sysUser);
    }
    /**
     * 更新用户
     * @param sysUser
     * @return boolean
     */
    @Override
    public boolean updateUser(SysUser sysUser){
        return this.updateById(sysUser);
    }
    /**
     * 删除用户
     * @param id
     * @return
     */
    @Override
    public boolean deleteUser(String id){
        return this.removeById(id);
    }
    /**
     * 根据账号查找
     * @param username
     * @return SysUser
     */
    @Override
    public SysUser queryUserName(String username){
        LambdaQueryWrapper<SysUser> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(SysUser::getUsername,username);
        SysUser sysUser = sysUserMapper.selectOne(queryWrapper);
        return sysUser;
    }
    /**
     * 根据邮箱查找
     * @param mailbox
     * @return SysUser
     */
    @Override
    public SysUser queryMailBox(String mailbox){
        LambdaQueryWrapper<SysUser> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(SysUser::getMailbox,mailbox);
        SysUser sysUser = sysUserMapper.selectOne(queryWrapper);
        return sysUser;
    }
    /**
     * 删除用户
     * @param id
     * @return
     */
    @Override
    public boolean removeUserById(Integer id) {
        LambdaUpdateWrapper<SysUser> updateWrapper = new UpdateWrapper<SysUser>().lambda();
        updateWrapper.eq(SysUser::getId, id).set(SysUser::getDelFlag, 1);
        int count = sysUserMapper.update(new SysUser(), updateWrapper);
        if (count > 0) {
            return true;
        }
        return false;
    }
    /**
     * 初始/重置密码
     * @param id            主键id
     * @param initPassword  初始密码
     * @return
     */
    @Override
    public boolean resetPassword(Integer id,String initPassword) {
        LambdaUpdateWrapper<SysUser> updateWrapper = new UpdateWrapper<SysUser>().lambda();
        updateWrapper.eq(SysUser::getId, id).set(SysUser::getLastUpdatePwdTime, null).set(SysUser::getPassword,initPassword);
        int count = sysUserMapper.update(new SysUser(), updateWrapper);
        if (count > 0) {
            return true;
        }
        return false;
    }
    /**
     * 解锁用户
     * @param id
     * @return
     */
    @Override
    public boolean relockUserById(Integer id) {
        LambdaUpdateWrapper<SysUser> updateWrapper = new UpdateWrapper<SysUser>().lambda();
        updateWrapper.eq(SysUser::getId, id).set(SysUser::getStatus, 0);
        int count = sysUserMapper.update(new SysUser(), updateWrapper);
        if (count > 0) {
            return true;
        }
        return false;
    }
}
