import router from './router'
import store from '@/store'
import { Message } from 'element-ui'
import NProgress from 'nprogress' // progress bar
import 'nprogress/nprogress.css' // progress bar style
import { getBrowserToken } from './utils/commonFuc' // get token from cookie
import getPageTitle from '@/utils/get-page-title'
import {doEncrypt, doDecryptStr,doSign,doVerifySignature,userinfoEncrypt,userinfoDecrypt} from '@/utils/smutil'

NProgress.configure({ showSpinner: false }) // NProgress Configuration

const whiteList = ['/login'] // no redirect whitelist

router.beforeEach(async (to, from, next) => {
  // start progress bar
  NProgress.start()

  // set page title
  document.title = getPageTitle(to.meta.title)
  if (sessionStorage.getItem('token')) {
    if (to.path === '/login') {
      // if is logged in, redirect to the home page
      next({ path: '/' })
      NProgress.done()
    } else {
      // 登录成功时将路由置空
      if (store.getters.permission_routes.length==0) {
        await store.dispatch('GenerateRoutes').then(accessRoutes => {
          // 根据roles权限生成可访问的路由表
          router.addRoutes(accessRoutes) // 动态添加可访问路由表
          next({ ...to, replace: true }) // hack方法 确保addRoutes已完成
        }).catch(err => {
          store.dispatch('changeRouters',[])
          console.log('生成路由异常，输出routes:'+store.getters.permission_routes.length)
          next({ path: '/' })
        })
      }

      // 验证初始密码是否修改了或者超过30天
      if (to.path !=='/dashboard' && to.path !=='/user/profile'){
        // 用户信息解密
        // let decryptUserInfo = userinfoDecrypt(store.getters.userinfo)
        let decryptUserInfo = userinfoDecrypt(sessionStorage.getItem('userinfo'))
        let user = JSON.parse(decryptUserInfo).sysUser
        if (user.lastUpdatePwdTime==null){
          Message({
            type: 'warning',
            message: '需要修改初始密码!'
          });
          next({ path: '/user/profile' })
        }
        else{
          var dateBegin = new Date(user.lastUpdatePwdTime);
          var dateEnd = new Date();
          var dateDiff = dateEnd.getTime() - dateBegin.getTime(); //时间差的毫秒数
          var dayDiff = Math.floor(dateDiff / (24 * 3600 * 1000)); //计算出相差天数
          if (dayDiff>=30){
            Message({
              type: 'warning',
              message: '密码超过30天需要修改!'
            })
            next({ path: '/user/profile' })
          }
          else{
            next()
          }
        }
      }
      else{
        next()
      }
    }
  } else {
    // 没有token
    if (whiteList.indexOf(to.path) !== -1) {
      // 在免登录白名单，直接进入
      next()
    } else {
      if (to.path === '/404') {
        next()
      }
      else{
        // 否则全部重定向到登录页
        next(`/login?redirect=${to.path}`)
        NProgress.done()
      }
    }
  }
})

router.afterEach(() => {
  // finish progress bar
  NProgress.done()
})
